@@ -31,6 +31,7 @@ int to_unicode (short int *charset, int c) {
 /************************************************************************/
 int from_unicode (CHARSET charset, int u) {
 	short int *p;
+	/* This is really assignment, not comparation */
 	if ((p=charset[(unsigned)u>>8])) {
 		return p[u & 0xff];
 	} else {
@@ -127,18 +128,18 @@ int get_8bit_char (FILE *f,long *offset,long fileend)
 /************************************************************************/
 
 int get_utf16lsb (FILE *f,long *offset,long fileend) {
-	char buf[2];
-	int d,c;
-
-	fread(buf, 1, 2, f);
-	c=(char)buf[0];
-	d=(char)buf[1];
-	if (c == EOF) return EOF;
-	if (d == EOF) return EOF;
-	c |= (d<<8);
+	unsigned char buf[2];
+    int result;
+	result=catdoc_read(buf, 1, 2, f);
+	if (result<0) {
+		perror("read:");
+		exit(1);
+	}
+	if (result !=2) {
+		return EOF;
+	}	
 	(*offset)+=2;
-	if (c == EOF) return (int)0xFEFF;
-	return c;
+	return ((int)buf[1])|(((int)buf[0])<<8);
 }
 
 /************************************************************************/
@@ -146,41 +147,45 @@ int get_utf16lsb (FILE *f,long *offset,long fileend) {
  * non-word files            .                                          */
 /************************************************************************/
 int get_utf16msb (FILE *f,long *offset,long fileend) {
-	char buf[2];
-	int d,c;
-
-	fread(buf, 1, 2, f);
-	c=(char)buf[0];
-	d=(char)buf[1];
-	if (c == EOF) return EOF;
-	if (d == EOF) return EOF;
-	c |= (d<<8);
+	unsigned char buf[2];
+    int result;
+	result=catdoc_read(buf, 1, 2, f);
+	if (result<0) {
+		perror("read:");
+		exit(1);
+	}
+	if (result !=2) {
+		return EOF;
+	}	
 	(*offset)+=2;
-	if (c == EOF) return (int)0xFEFF;
-	return c;
+	return ((int)buf[0])|(((int)buf[1])<<8);
 }
 
 int get_utf8 (FILE *f,long *offset,long fileend) {
-	char buf[3];
+	unsigned char buf[3];
 	int d,c;
-
-	fread(buf, 1, 1, f);
-	c=(unsigned char)buf[0];
+    int result;
+	result=catdoc_read(buf, 1, 1, f);
+	if (result<0) {
+		perror("read");
+		exit(1);
+	}	
+	if (result==0) return EOF;
+	c=buf[0];
 	d=0;
-	if (c == EOF) return EOF;
 	if (c<0x80) 
 		return c;
 	if (c <0xC0) 
 		return 0xfeff; /*skip corrupted sequebces*/
 	if (c <0xE0) {
-		fread(buf+1, 1, 1, f);
+		if (catdoc_read(buf+1, 1, 1, f)<=0) return EOF;
 		return ((c & 0x1F)<<6 | ((char)buf[1] & 0x3F));
 	}
 	if (c <0xF0) {
-		fread(buf+1, 1, 2, f);
+		if (catdoc_read(buf+1, 1, 2, f)<=2) return (int)EOF;
 		return ((c & 0x0F)<<12)|
-			(((char)buf[1] & 0x3f)<<6)|
-					 ((char)buf[2] & 0x3f);
+			((buf[1] & 0x3f)<<6)|
+					 (buf[2] & 0x3f);
 	}  
 	return 0xFEFF; 
 }
@@ -243,3 +248,55 @@ char *to_utf8(unsigned int uc) {
 	utfbuffer[count]=0;
 	return utfbuffer;
 }    
+
+struct cp_map {
+	int codepage;
+	char *charset_name;
+};
+
+struct cp_map cp_to_charset [] = {
+	{10000,"mac-roman"},
+	{10001,"mac-japanese"},
+	{10002,"mac-tchinese"},
+	{10003,"mac-korean"},
+	{10004,"mac-arabic"},
+	{10005,"mac-hebrew"},
+	{10006,"mac-greek1"},
+	{10007,"mac-cyrillic"},
+	{10008,"mac-schinese"},
+	{10010,"mac-romania"},
+	{10017,"mac-ukraine"},
+	{10021,"mac-thai"},
+	{10029,"mac-centeuro"},
+	{10079,"mac-iselandic"},
+	{10081,"mac-turkish"},
+	{10082,"mac-croatia"},
+	{20866,"koi8-r"},
+	{28591,"8859-1"},
+	{28592,"8859-2"},
+	{28593,"8859-3"},
+	{28594,"8859-4"},
+	{28595,"8859-5"},
+	{28596,"8859-6"},
+	{28597,"8859-7"},
+	{28598,"8859-8"},
+	{28599,"8859-9"},
+	{28605,"8859-15"},
+	{65001,"utf-8"},
+    {0,NULL}};
+const char *charset_from_codepage(unsigned int codepage) {
+	
+	static char buffer[7];
+	struct cp_map *cp;
+	if (codepage==1200||codepage==1201) {
+		/* For UCS2 */
+		return "";
+	} else 
+	if (codepage<10000) {
+		sprintf(buffer,"cp%d",codepage);
+		return buffer;
+	} else {
+		for (cp = cp_to_charset;cp->codepage!=0&& cp->codepage!=codepage;cp++);
+		return cp->charset_name;
+	}
+}	
