@@ -2,7 +2,7 @@
  * @file   ole.c
  * @author Alex Ott, Victor B Wagner
  * @date   Wed Jun 11 12:33:01 2003
- * Version: $Id: ole.c,v 1.14 2003/12/24 12:20:46 ott Exp $
+ * Version: $Id: ole.c,v 1.19 2005/05/01 10:55:09 vitus Exp $
  * Copyright: Victor B Wagner, 1996-2003 Alex Ott, 2003
  * 
  * @brief  Parsing structure of MS Office compound document
@@ -24,7 +24,7 @@
 
 #define min(a,b) ((a) < (b) ? (a) : (b))
 
-long int sectorSize;
+long int sectorSize, shortSectorSize;
 /* BBD Info */
 long int  bbdStart, bbdNumBlocks;
 unsigned char *BBD=NULL;
@@ -39,6 +39,7 @@ long int fileLength=0;
 
 static char ole_sign[]={0xD0,0xCF,0x11,0xE0,0xA1,0xB1,0x1A,0xE1,0};
 
+
 /** 
  * Initializes ole structure
  * 
@@ -52,9 +53,10 @@ static char ole_sign[]={0xD0,0xCF,0x11,0xE0,0xA1,0xB1,0x1A,0xE1,0};
  */
 FILE* ole_init(FILE *f, void *buffer, size_t bufSize)  {
 	unsigned char oleBuf[BBD_BLOCK_SIZE];
+	unsigned char *tmpBuf;
 	FILE *newfile;
 	int ret=0, i;
-	long int sbdMaxLen, sbdCurrent, propMaxLen, propCurrent;
+	long int sbdMaxLen, sbdCurrent, propMaxLen, propCurrent, mblock, msat_size;
 	oleEntry *tEntry;
 
 	/* deleting old data (if it was allocated) */
@@ -83,6 +85,7 @@ FILE* ole_init(FILE *f, void *buffer, size_t bufSize)  {
 	}	
 	fseek(newfile,0,SEEK_END);
 	fileLength=ftell(newfile);
+/* 	fprintf(stderr, "fileLength=%ld\n", fileLength);    */
 	fseek(newfile,0,SEEK_SET);
 	ret=fread(oleBuf,1,BBD_BLOCK_SIZE,newfile);
 	if ( ret != BBD_BLOCK_SIZE ) {
@@ -91,33 +94,64 @@ FILE* ole_init(FILE *f, void *buffer, size_t bufSize)  {
 	if (strncmp(oleBuf,ole_sign,8) != 0) {
 		return NULL;
 	}
-/* 	if ( (sectorSize = BBD_BLOCK_SIZE * getlong(oleBuf,0x40)) == 0) */
-	sectorSize = BBD_BLOCK_SIZE;
+ 	sectorSize = 1<<getshort(oleBuf,0x1e);
+	shortSectorSize=1<<getshort(oleBuf,0x20);
 	
 /* Read BBD into memory */
 	bbdStart=getlong(oleBuf,0x4c);
 	bbdNumBlocks = getulong(oleBuf,0x2c);
-	if((BBD=malloc(bbdNumBlocks*BBD_BLOCK_SIZE)) == NULL ) {
+	if((BBD=malloc(bbdNumBlocks*sectorSize)) == NULL ) {
 		return NULL;
 	}
+	
+	if((tmpBuf=malloc(MSAT_ORIG_SIZE)) == NULL ) {
+		return NULL;
+	}
+	memcpy(tmpBuf,oleBuf+0x4c,MSAT_ORIG_SIZE);
+	mblock=getlong(oleBuf,0x44);
+	msat_size=getulong(oleBuf,0x48);
+	i=0;
+/* 	fprintf(stderr, "i=%d mblock=%ld msat_size=%ld\n", i, mblock, msat_size); */
+	
+	while((mblock >= 0) && (i < msat_size)) {
+		char *newbuf;
+/* 		fprintf(stderr, "i=%d mblock=%ld\n", i, mblock); */
+		if ((newbuf=realloc(tmpBuf, sectorSize*(i+1)+MSAT_ORIG_SIZE)) != NULL) {
+			tmpBuf=newbuf;
+		} else {
+			perror("BDB realloc error");
+			free(tmpBuf);
+			ole_finish();
+			return NULL;
+		}
+		
+		fseek(newfile, 512+mblock*sectorSize, SEEK_SET);
+		fread(tmpBuf+MSAT_ORIG_SIZE+sectorSize*i-(i==0 ? 0 : (4*i)),
+					1, sectorSize, newfile);
+		i++;
+		mblock=getlong(tmpBuf, MSAT_ORIG_SIZE+sectorSize*i-4);
+	};
+	
 /*   	fprintf(stderr, "bbdNumBlocks=%ld\n", bbdNumBlocks);   */
 	for(i=0; i< bbdNumBlocks; i++) {
-		long int bbdSector=getlong(oleBuf,0x4c+4*i);
+		long int bbdSector=getlong(tmpBuf,4*i);
 		
-		if (bbdSector >= fileLength/sectorSize) {
+/*  		fprintf(stderr, "bbdSector(%d)=%ld\n",i,bbdSector); */
+		if (bbdSector >= fileLength/sectorSize || bbdSector < 0) {
 			fprintf(stderr, "Bad BBD entry!\n");
 			ole_finish();
 			return NULL;
 		}
-/*  		fprintf(stderr, "bbdSector=%ld\n",bbdSector);  */
-		fseek(newfile, (bbdSector+1)*BBD_BLOCK_SIZE, SEEK_SET);
-		if ( fread(BBD+i*BBD_BLOCK_SIZE, 1, BBD_BLOCK_SIZE, newfile) !=
-			 BBD_BLOCK_SIZE ) {
+		fseek(newfile, 512+bbdSector*sectorSize, SEEK_SET);
+		if ( fread(BBD+i*sectorSize, 1, sectorSize, newfile) != sectorSize ) {
 			fprintf(stderr, "Can't read BBD!\n");
+			free(tmpBuf);
 			ole_finish();
 			return NULL;
 		}
 	}
+	free(tmpBuf);
+	
 /* Read SBD into memory */
 	sbdLen=0;
 	sbdMaxLen=10;
@@ -128,8 +162,8 @@ FILE* ole_init(FILE *f, void *buffer, size_t bufSize)  {
 			return NULL;
 		}
 		while(1) {
-			fseek(newfile, (sbdCurrent+1)*sectorSize, SEEK_SET);
-			fread(SBD+sbdLen*BBD_BLOCK_SIZE, 1, sectorSize, newfile);
+			fseek(newfile, 512+sbdCurrent*sectorSize, SEEK_SET);
+			fread(SBD+sbdLen*sectorSize, 1, sectorSize, newfile);
 			sbdLen++;
 			if (sbdLen >= sbdMaxLen) {
 				char *newSBD;
@@ -148,7 +182,7 @@ FILE* ole_init(FILE *f, void *buffer, size_t bufSize)  {
 				sbdCurrent >= fileLength/sectorSize)
 				break;
 		}
-		sbdNumber = (sbdLen*sectorSize)/SBD_BLOCK_SIZE;
+		sbdNumber = (sbdLen*sectorSize)/shortSectorSize;
 /*   		fprintf(stderr, "sbdLen=%ld sbdNumber=%ld\n",sbdLen, sbdNumber); */
 	} else {
 		SBD=NULL;
@@ -164,7 +198,7 @@ FILE* ole_init(FILE *f, void *buffer, size_t bufSize)  {
 		}
 		while(1) {
 /*  			fprintf(stderr, "propCurrent=%ld\n",propCurrent); */
-			fseek(newfile, (propCurrent+1)*sectorSize, SEEK_SET);
+			fseek(newfile, 512+propCurrent*sectorSize, SEEK_SET);
 			fread(properties+propLen*sectorSize,
 				  1, sectorSize, newfile);
 			propLen++;
@@ -277,9 +311,9 @@ FILE *ole_readdir(FILE *f) {
 /* 	fprintf(stderr, "e->name=%s e->length=%ld\n", e->name, e->length); */
 /* 	fprintf(stderr, "e->startBlock=%ld BBD=%p\n", e->startBlock, BBD); */
 	if (e->startBlock >= 0 &&
-		e->length > 0 &&
+		e->length >= 0 &&
 		(e->startBlock <=
-		 fileLength/(e->isBigBlock ? sectorSize : SBD_BLOCK_SIZE))) {
+		 fileLength/(e->isBigBlock ? sectorSize : shortSectorSize))) {
 		if((e->blocks=malloc(chainMaxLen*sizeof(long int))) == NULL ) {
 			return NULL;
 		}
@@ -308,20 +342,20 @@ FILE *ole_readdir(FILE *f) {
 			}
 			if(chainCurrent <= 0 ||
 			   chainCurrent >= ( e->isBigBlock ?
-								 ((bbdNumBlocks*BBD_BLOCK_SIZE)/4)
-								 : ((sbdNumber*SBD_BLOCK_SIZE)/4) ) ||
+								 ((bbdNumBlocks*sectorSize)/4)
+								 : ((sbdNumber*shortSectorSize)/4) ) ||
 			   (e->numOfBlocks >
-				e->length/(e->isBigBlock ? sectorSize : SBD_BLOCK_SIZE))) {
+				e->length/(e->isBigBlock ? sectorSize : shortSectorSize))) {
 /*   				fprintf(stderr, "chain End=%ld\n", chainCurrent);   */
 				break;
 			}
-			
 		}
 	}
-	if(e->length > (e->isBigBlock ? sectorSize : SBD_BLOCK_SIZE)*e->numOfBlocks)
-		e->length = (e->isBigBlock ? sectorSize : SBD_BLOCK_SIZE)*e->numOfBlocks;
-/*	fprintf(stderr, "READDIR: e->name=%s e->numOfBlocks=%ld length=%ld\n",
-	e->name, e->numOfBlocks, e->length);*/
+	
+	if(e->length > (e->isBigBlock ? sectorSize : shortSectorSize)*e->numOfBlocks)
+		e->length = (e->isBigBlock ? sectorSize : shortSectorSize)*e->numOfBlocks;
+/* 	fprintf(stderr, "READDIR: e->name=%s e->numOfBlocks=%ld length=%ld\n", */
+/* 					e->name, e->numOfBlocks, e->length); */
 	
 	return (FILE*)e;
 }
@@ -338,8 +372,8 @@ int ole_open(FILE *stream) {
 	if ( e->type != oleStream)
 		return -2;
 	
-	e->readed=0;
-	e->offset= ftell(e->file);
+	e->ole_offset=0;
+	e->file_offset= ftell(e->file);
 	return 0;
 }
 
@@ -354,13 +388,14 @@ int ole_open(FILE *stream) {
 long int calcFileBlockOffset(oleEntry *e, long int blk) {
 	long int res;
 	if ( e->isBigBlock ) {
-		res=(e->blocks[blk]+1)*sectorSize;
+		res=512+e->blocks[blk]*sectorSize;
 	} else {
-		long int sbdPerSector=sectorSize/SBD_BLOCK_SIZE;
+		long int sbdPerSector=sectorSize/shortSectorSize;
 		long int sbdSecNum=e->blocks[blk]/sbdPerSector;
 		long int sbdSecMod=e->blocks[blk]%sbdPerSector;
-		res=(rootEntry->blocks[sbdSecNum]+1)*sectorSize +
-			sbdSecMod*SBD_BLOCK_SIZE;
+/* 		fprintf(stderr, "calcoffset: e->name=%s e->numOfBlocks=%ld length=%ld sbdSecNum=%ld rootEntry->blocks=%p\n", */
+/* 						e->name, e->numOfBlocks, e->length, sbdSecNum, rootEntry->blocks); */
+		res=512 + rootEntry->blocks[sbdSecNum]*sectorSize + sbdSecMod*shortSectorSize;
 	}
 	return res;
 }
@@ -383,17 +418,17 @@ size_t ole_read(void *ptr, size_t size, size_t nmemb, FILE *stream) {
 	long int ssize;				/**< Size of block */
 	long int newoffset;
 	unsigned char *cptr = ptr;	
-	if( e->readed+llen > e->length )
-		llen= e->length - e->readed;
+	if( e->ole_offset+llen > e->length )
+		llen= e->length - e->ole_offset;
 	
-	ssize = (e->isBigBlock ? sectorSize : SBD_BLOCK_SIZE);
-	blockNumber=e->readed/ssize;
+	ssize = (e->isBigBlock ? sectorSize : shortSectorSize);
+	blockNumber=e->ole_offset/ssize;
 /* 	fprintf(stderr, "blockNumber=%ld e->numOfBlocks=%ld llen=%ld\n", */
 /* 			blockNumber, e->numOfBlocks, llen); */
 	if ( blockNumber >= e->numOfBlocks || llen <=0 )
 		return 0;
 	
-	modBlock=e->readed%ssize;
+	modBlock=e->ole_offset%ssize;
 	bytesInBlock = ssize - modBlock;
 	if(bytesInBlock < llen) {
 		toReadBlocks = (llen-bytesInBlock)/ssize;
@@ -404,33 +439,33 @@ size_t ole_read(void *ptr, size_t size, size_t nmemb, FILE *stream) {
 /* 	fprintf(stderr, "llen=%ld toReadBlocks=%ld toReadBytes=%ld bytesInBlock=%ld blockNumber=%ld modBlock=%ld\n", */
 /* 			llen, toReadBlocks, toReadBytes, bytesInBlock, blockNumber, modBlock); */
 	newoffset = calcFileBlockOffset(e,blockNumber)+modBlock;
-	if (e->offset != newoffset) {
-		fseek(e->file, e->offset=newoffset, SEEK_SET);
+	if (e->file_offset != newoffset) {
+		fseek(e->file, e->file_offset=newoffset, SEEK_SET);
 	}
 	rread=fread(ptr, 1, min(llen,bytesInBlock), e->file);
-	e->offset += rread;
+	e->file_offset += rread;
 	for(i=0; i<toReadBlocks; i++) {
 		int readbytes;
 		blockNumber++;
 		newoffset = calcFileBlockOffset(e,blockNumber);
-		if (newoffset != e->offset);
-		fseek(e->file, e->offset=newoffset , SEEK_SET);
+		if (newoffset != e->file_offset);
+		fseek(e->file, e->file_offset=newoffset , SEEK_SET);
 		readbytes=fread(cptr+rread, 1, min(llen-rread, ssize), e->file);
 		rread +=readbytes;
-		e->offset +=readbytes;
+		e->file_offset +=readbytes;
 	}
 	if(toReadBytes > 0) {
 		int readbytes;
 		blockNumber++;
 		newoffset = calcFileBlockOffset(e,blockNumber);
-		fseek(e->file, e->offset=newoffset, SEEK_SET);
+		fseek(e->file, e->file_offset=newoffset, SEEK_SET);
         readbytes=fread(cptr+rread, 1, toReadBytes,e ->file);
 		rread +=readbytes;
-		e->offset +=readbytes;
+		e->file_offset +=readbytes;
 	}
-/*	fprintf(stderr, "readed=%ld rread=%ld llen=%ld\n",
-	e->readed, rread, llen);*/
-	e->readed+=rread;
+/*	fprintf(stderr, "ole_offset=%ld rread=%ld llen=%ld\n",
+	e->ole_offset, rread, llen);*/
+	e->ole_offset+=rread;
 	return rread;
 }	
 
@@ -443,9 +478,9 @@ size_t ole_read(void *ptr, size_t size, size_t nmemb, FILE *stream) {
  */
 int ole_eof(FILE *stream) {
 	oleEntry *e=(oleEntry*)stream;
-/*	fprintf(stderr, "EOF: e->readed=%ld  e->length=%ld\n",
-	e->readed,  e->length);*/
-	return (e->readed >=  e->length);
+/*	fprintf(stderr, "EOF: e->ole_offset=%ld  e->length=%ld\n",
+	e->ole_offset,  e->length);*/
+	return (e->ole_offset >=  e->length);
 }
 
 /** 
@@ -478,16 +513,82 @@ int ole_close(FILE *stream) {
 	return 0;
 }
 
+/**
+ * 
+ * 
+ * @param stream pointer to OLE stream structure
+ * @param offset 
+ * @param whence 
+ * 
+ * @return 
+ */
+int ole_seek(FILE *stream, long offset, int whence) {
+	oleEntry *e=(oleEntry*)stream;
+	long int new_ole_offset=0, new_file_offset;
+	int ssize, modBlock, blockNumber;
+	
+	switch(whence) {
+	case SEEK_SET:
+		new_ole_offset=offset;
+		break;
+		
+	case SEEK_CUR:
+		new_ole_offset=e->ole_offset+offset;
+		break;
+		
+	case SEEK_END:
+		new_ole_offset=e->length+offset;
+		break;
+		
+	default:
+		errno=EINVAL;
+		return -1;
+	}
+	if(new_ole_offset<0)
+		new_ole_offset=0;
+	if(new_ole_offset >= e->length)
+		new_ole_offset=e->length;
+
+	ssize = (e->isBigBlock ? sectorSize : shortSectorSize);
+	blockNumber=new_ole_offset/ssize;
+	if ( blockNumber >= e->numOfBlocks )
+		return -1;
+	
+	modBlock=new_ole_offset%ssize;
+	new_file_offset = calcFileBlockOffset(e,blockNumber)+modBlock;
+	fseek(e->file, e->file_offset=new_file_offset, SEEK_SET);
+	e->ole_offset=new_ole_offset;
+	
+	return 0;
+}
+
+/** 
+ * Tell position inside OLE stream
+ * 
+ * @param stream pointer to OLE stream
+ * 
+ * @return current position inside OLE stream
+ */
+long ole_tell(FILE *stream) {
+	oleEntry *e=(oleEntry*)stream;
+	return e->ole_offset;
+}
+
+
 /**
  * 
  * 
  */
 size_t (*catdoc_read)(void *ptr, size_t size, size_t nmemb, FILE *stream);
 int (*catdoc_eof)(FILE *stream);
+int (*catdoc_seek)(FILE *stream, long offset, int whence);
+long (*catdoc_tell)(FILE *stream);
 
 void set_ole_func(void) {
 	catdoc_read=ole_read;
 	catdoc_eof=ole_eof;
+	catdoc_seek=ole_seek;
+	catdoc_tell=ole_tell;
 }
 
 #ifdef feof
@@ -505,5 +606,7 @@ int my_feof(FILE *f) {
 void set_std_func(void) {
 	catdoc_read=fread;
 	catdoc_eof=FEOF;
+	catdoc_seek=fseek;
+	catdoc_tell=ftell;
 }
 
