@@ -2,7 +2,7 @@
  * @file   ole.h
  * @author Alex Ott <ott@jet.msk.su>
  * @date   03  2003
- * Version: $Id: ole.h,v 1.3 2003/11/14 14:32:39 vitus Exp $
+ * Version: $Id: ole.h,v 1.5 2005/03/15 20:06:51 vitus Exp $
  * Copyright: Alex Ott, 2003, 
  * 
  * @brief  
@@ -17,6 +17,7 @@
 #define SBD_BLOCK_SIZE      64
 #define PROP_BLOCK_SIZE    128
 #define OLENAMELENGHT       32
+#define MSAT_ORIG_SIZE     436
 
 typedef enum {
 	oleDir=1,
@@ -28,13 +29,11 @@ typedef enum {
 typedef struct {
 	FILE *file;
 	char name[OLENAMELENGHT+1];
-	long int startOffset;
-	long int curOffset;
 	long int startBlock;
 	long int curBlock;
 	unsigned long int length;
-	long int readed;
-	long int offset;
+	long int ole_offset;
+	long int file_offset;
 	unsigned char *dirPos;
 	oleType type;
 	long int numOfBlocks;
@@ -48,6 +47,8 @@ typedef struct {
  */
 FILE* ole_init(FILE *f, void *buffer, size_t bufSize);
 FILE *ole_readdir(FILE *f);
+int ole_seek(FILE *stream, long offset, int whence);
+long ole_tell(FILE *stream);
 int ole_eof(FILE *stream);
 size_t ole_read(void *ptr, size_t size, size_t nmemb, FILE *stream);
 int ole_open(FILE *);
@@ -56,6 +57,9 @@ void ole_finish(void);
 
 extern size_t (*catdoc_read)(void *ptr, size_t size, size_t nmemb, FILE *stream);
 extern int (*catdoc_eof)(FILE *stream);
+extern int (*catdoc_seek)(FILE *stream, long offset, int whence);
+extern long (*catdoc_tell)(FILE *stream);
+
 void set_ole_func(void);
 void set_std_func(void);
 
