@@ -66,6 +66,7 @@ typedef struct {
 
 RTFTypeMap rtf_types[]={
 	{"uc",RTF_UC},
+	{"ansicpg",RTF_CODEPAGE},
 	{"pard",RTF_PARD},
 	{"par",RTF_PARA},
 	{"cell",RTF_CELL},
@@ -146,11 +147,15 @@ typedef struct {
  * Functions declaration
  * 
  */
-signed int getNumber(FILE *f);
+
+extern int forced_charset;
+signed long getNumber(FILE *f);
+
 int getRtfCommand(FILE *f, RTFcommand *command );
 unsigned short int rtf_to_unicode(int code);
 RTFTypes getCommandType(char *name);
 signed int getCharCode(FILE *f);
+void rtfSetCharset(short int **charset_ptr,unsigned int codepage);
 
 /********************************************************
  * Global data
@@ -280,6 +285,8 @@ int parse_rtf(FILE *f) {
 			case RTF_LANG:
 /* 				fprintf(stderr, "Selected lang = %d\n",com.numarg); */
 				break;
+			case RTF_CODEPAGE:
+				rtfSetCharset(&current_charset,com.numarg);
 			default:
 /*  				fprintf(stderr, "Unknown command with name %s and arg=%d\n",  */
 /*  						com.name, com.numarg);  */
@@ -339,7 +346,7 @@ int parse_rtf(FILE *f) {
  * 
  * @return converted number
  */
-signed int getNumber(FILE *f) {
+signed long getNumber(FILE *f) {
 	int c,count=0;
 	char buf[RTFARGSMAXLEN];
 	
@@ -453,3 +460,15 @@ signed int getCharCode(FILE *f) {
 	return strtol(buf, (char **)NULL, 16);
 }
 
+void rtfSetCharset(short int **charset_ptr,unsigned int codepage)
+{
+	/* Do not override charset if it is specified in the command line */
+	const char *charset_name;
+	char *save_buf = input_buffer;
+	if (forced_charset) return;
+	charset_name = charset_from_codepage(codepage);
+	check_charset(&source_csname,charset_name);
+	input_buffer=NULL;
+	*charset_ptr = read_charset(source_csname);	
+	input_buffer = save_buf;
+}
