@@ -25,15 +25,20 @@ short int *formatTable=NULL;
 char *forced_date_format = NULL;
 size_t formatTableIndex = 0;
 size_t formatTableSize = 0;
-
+double date_shift = 25569.0;
 #define FLT_FORMAT(a,b,c) a #b c
 #define MK_FORMAT(x) FLT_FORMAT("%.",x,"g") 			
 char number_format[8]=MK_FORMAT(DBL_DIG);
+
+void CleanUpFormatIdxUsed(void);
+
 void do_table(FILE *input,char *filename) {    
 	long rectype;
 	long reclen,build_year=0,build_rel=0,offset=0;
 	int eof_flag=0;
 	int itemsread=1;
+	date_shift=25569.0; /* Windows 1900 date system */
+	CleanUpFormatIdxUsed();
 	while (itemsread) {
 		catdoc_read(rec,2,1,input);
 		biff_version=getshort(rec,0);
@@ -143,28 +148,33 @@ void process_item (int rectype, int reclen, char *rec) {
 					   codepage=getshort(rec,0);
 					   /*fprintf(stderr,"CODEPAGE %d\n",codepage);*/ 
 					   if (codepage!=1200) {
-						   char cp[9];
-						   sprintf(cp,"cp%d",codepage);
+						   const char *cp = charset_from_codepage(codepage); 
 						   check_charset(&source_csname,cp);
 						   source_charset=read_charset(source_csname);
 					   }		 
 					   break;
 				   }  
 		case FORMAT: {
-						 /*     int i;
-								char *ptr;
-								printf( "Format %x \"",getshort(rec,0));	 
-								if (rec[2] == reclen - 3 && rec[3] != 0) {
-								for (i=0,ptr=rec+3;i<rec[2];i++,ptr++) {
-								fputc(*ptr,stdout);
-								}
-								} else {
-								for (i=0,ptr=rec+5;i<rec[2];i++,ptr+=2) {
-								fputc(*ptr,stdout);
-								}    
-								}	 
-								printf ("\"\n"); */
-						 break;
+						      int format_code;
+						      format_code=getshort(rec,0);
+						      SetFormatIdxUsed(format_code);
+ 						     /* this debug code prints format string */
+/*							  
+						      int i;
+						      char *ptr;
+ 						      fprintf(stderr,"Format %x \"",format_code);
+ 						      if (rec[2] == reclen - 3 && rec[3] != 0) {
+ 							for (i=0,ptr=rec+3;i<rec[2];i++,ptr++) {
+ 							  fputc(*ptr,stderr);
+ 							}
+ 						      } else {
+ 							for (i=0,ptr=rec+5;i<rec[2];i++,ptr+=2) {
+ 							  fputc(*ptr,stderr);
+ 							}
+ 						      }
+ 						      fprintf (stderr,"\"\n");
+*/
+							 break;
 					 }			 
 		case SST: {
 					  /* Just copy SST into buffer, and wait until we get
@@ -258,7 +268,6 @@ void process_item (int rectype, int reclen, char *rec) {
 		case 0x03:
 		case 0x103:
 		case 0x303:
-							  fprintf(stderr,"Something like number found: %X\n",rectype);		      
 		case NUMBER: {
 						 int row,col;
 						 unsigned char **pcell;
@@ -369,6 +378,10 @@ void process_item (int rectype, int reclen, char *rec) {
 					  formatTable[formatTableIndex++] = formatIndex;
 					  break;
 				  } 
+		case MS1904: /* Macintosh 1904 date system */ 
+				    date_shift=24107.0; 
+					break;	 
+						 
 					 
 		case MSEOF: {
 						if (!rowptr) break;
@@ -506,6 +519,109 @@ char *copy_unicode_string (unsigned char **src) {
 	*src=s+to_skip;
 	return dest;
 }
+
+
+/* 
+ * Format code is index into format table (which is list of XF records
+ * in the file 
+ * Second word of XF record is format type idnex 
+ * format index between 0x0E and 0x16 also between 0x2D and ox2F denotes
+ * date if it is not used for explicitly stored formats.
+ * BuiltInDateFormatIdx converts format index into index of explicit
+ * built-in date formats sutable for strftime.
+ */
+int BuiltInDateFormatIdx (int index) {
+	int offset;
+	offset=1; /* offset of date formats */
+	/* 0 is used as false -- format not found */
+	if ((index>= 0x0E) && (index<=0x16)) {
+			return offset+index-0x0E;
+	} else 	
+	if ((index>=0x2d) && (index<=0x2F)) {
+		return offset+index-0x2d+9;
+	} else if (index==0xa4) {	
+		return 12+offset;
+	} else 	
+	return 0;
+}	
+
+/* 
+ * GetBuiltInDateFormat stores and returns
+ * built in xls2csv strftime formats.
+ */
+#define NUMOFDATEFORMATS 13
+char *GetBuiltInDateFormat(int dateindex) {
+	static char *formats[]={
+	/* reserved  */ NULL, /* BuiltInDateFormatIdx use dateindex=0 as flag format not found */
+	/* 0x0E */ "%m-%d-%y",		/* 01 */
+	/* 0x0F */ "%d-%b-%y",		/* 02 */
+	/* 0x10 */ "%d-%b",		/* 03 */
+	/* 0x11 */ "%b-%d",	 	/* 04 */
+	/* 0x12 */ "%l:%M %p",		/* 05 */
+	/* 0x13 */ "%l:%M:%S %p",		/* 06 */
+	/* 0x14 */ "%H:%M",		/* 07 */
+	/* 0x15 */ "%H:%M:%S",		/* 08 */
+	/* 0x16 */ "%m-%d-%y %H:%M",	/* 09 */
+	/* 0x2d */ "%M:%S",		/* 10 */
+	/* 0x2e */ "%H:%M:%S",		/* 11 */
+	/* 0x2f */ "%M:%S",		/* 12 */
+	/* 0xa4 */ "%m.%d.%Y %l:%M:%S %p"	/* 13 */
+	};
+	if (dateindex>0 && dateindex <= NUMOFDATEFORMATS) {
+	  return formats[dateindex];
+	} else 	
+	return NULL;
+}	
+
+static char FormatIdxUsed[NUMOFDATEFORMATS];
+
+void CleanUpFormatIdxUsed() {
+	int i;
+	for (i=0;i<NUMOFDATEFORMATS; i++);
+	FormatIdxUsed[i]=0;
+}
+
+/* 
+ * format index between 0x0E and 0x16 also between 0x2D and ox2F denotes
+ * date in case when they are built-in Excel97 formats.
+ * Nevertheless, those indexes can be used for explicitly stored formats,
+ * which are not dates in general.
+ * SetFormatIdxUsed marks this formats as already used 
+ * and excludes them from list of built-in formats 
+ * preventing misformatting of corresponding data.
+ */
+void SetFormatIdxUsed(int format_code) {
+	int dateindex;
+ 	/*fprintf(stderr,"Format idx %x to be set to dirty\n",format_code);
+	 */
+        dateindex=BuiltInDateFormatIdx(format_code);
+	if (dateindex) {
+	  FormatIdxUsed[dateindex]=1;
+ 	  /*fprintf(stderr,"Date idx %d is set to be dirty\n",dateindex); */
+	}
+}	
+
+/* 
+ * format index between 0x0E and 0x16 also between 0x2D and ox2F denotes
+ * date in case when they are built-in Excel97 formats.
+ * Nevertheless, those indexes can be used for explicitly stored formats,
+ * which are not dates in general.
+ * SetFormatIdxUsed marks this formats as already used 
+ * and excludes them from list of built-in formats 
+ * preventing misformatting of corresponding data.
+ * IsFormatIdxUsed tests this case.
+ */
+char IsFormatIdxUsed(int format_code) {
+	int dateindex;
+        dateindex=BuiltInDateFormatIdx(format_code);
+	if (dateindex) {
+/* 	  fprintf(stderr,"Date idx %d is dirty\n",dateindex); */
+	  return FormatIdxUsed[dateindex]==1;
+	}
+	else return 0;
+}	
+
+
 /* Checks if format denoted by given code is date
  * Format code is index into format table (which is list of XF records
  * in the file 
@@ -515,41 +631,32 @@ char *copy_unicode_string (unsigned char **src) {
  * If so, it returns strftime format for this date. Otherwise returns
  * NULL
  */
+
+
 char *isDateFormat(int format_code) {
 	int index;
-static char *formats[]={
-	/* 0x0E */ "%m-%d-%y",
-	/* 0x0F */ "%d-%b-%y",
-	/* 0x10 */ "%d-%b",
-	/* 0x11 */ "%b-%d", 
-	/* 0x12 */ "%l:%M %p",
-	/* 0x13 */ "%l:%M:%S %p",
-	/* 0x14 */ "%H:%M",
-	/* 0x15 */ "%H:%M:%S",
-	/* 0x16 */ "%m-%d-%y %H:%M",
-	/* 0x2d */ "%M:%S",
-	/* 0x2e */ "%H:%M:%S",
-	/* 0x2f */ "%M:%S",
-	/* 0xa4 */ "%m.%d.%Y %l:%M:%S %p"
-};
+	int dateindex;
 	if (format_code>=formatTableIndex) {
 		fprintf(stderr,"Format code %d is used before definition\n",format_code);
 		return NULL;
 	}
+	
 	index = formatTable[format_code];
-	if ((index>= 0x0E) && (index<=0x16)) {
-		if (forced_date_format) return forced_date_format;
-			return formats[index-0x0E];
-	} else 	
-	if ((index>=0x2d) && (index<=0x2F)) {
-		if (forced_date_format) return forced_date_format;
-		return formats[index-0x2d+9];
-	} else if (index==0xa4) {	
-		if (forced_date_format) return forced_date_format;
-		return formats[12];
+	if (IsFormatIdxUsed(index)) {
+	   fprintf(stderr,"Format %x is redefined\n",index);
+	  /* this format is something user-defined --- not a standard built-in date*/
+	  return NULL;
+	}
+        dateindex=BuiltInDateFormatIdx(index);
+	if (dateindex) {
+	  if (forced_date_format) return forced_date_format;
+			return GetBuiltInDateFormat(dateindex);
 	} else 	
 	return NULL;
 }	
+
+
+
 time_t float2date(double d);
 /*
  * Extracts floating point value and formats it 
@@ -629,10 +736,11 @@ char* format_rk(char *rec,short int format_code) {
  */
 time_t float2date(double f) { 
 	/* Hacked version. Excell stores date as floating point count of days
-	 * since 1.1.1900. We are substracting value of 1.1.1970 and multiplying
+	 * since 1.1.1900. or 1.1.1904
+	 * We are substracting value of 1.1.1970 and multiplying
 	 * by 86400 thus getting seconds from the epoch
 	 */
-	return rint((f-25569.0)*86400); 
+	return rint((f-date_shift)*86400); 
 }
 
 /*
